Package: batchCorr
Type: Package
Title: Within And Between Batch Correction Of LC-MS Metabolomics Data
Version: 1.1.0
Date: 2020-05-29
Authors@R: c(
    person("Anton", "Ribbenstedt", email = "parasitetwin@gmail.com", role = c("cre"), comment = c(ORCID = "0000-0002-9985-5644")),
    person("Carl", "Brunius", email = "carl.brunius@chalmers.se", role = c("aut"), comment = c(ORCID = "0000-0003-3957-870X")),
    person("Vilhelm", "Suksi", role = c("aut")))
Description: From the perspective of metabolites as the continuation of the central dogma of biology, metabolomics provides the closest link to many phenotypes of interest. This makes metabolomics research promising in teasing apart the complexities of living systems. However, due to experimental reasons, the data includes non-biological variation which limits quality and reproducibility, especially if the data is obtained from several batches. The batchCorr package reduces unwanted variation by way of between-batch alignment, within-batch drift correction and between-batch normalization using batch-specific quality control samples and long-term reference QC samples. Please see the associated article for more thorough descriptions of algorithms.
License: GPL-2
biocViews: BiomedicalInformatics, Metabolomics, MassSpectrometry,
        BatchEffect, Normalization, QualityControl
Depends: R (>= 4.4.0), SummarizedExperiment
Imports: reshape, mclust, BiocParallel, methods
Suggests: BiocStyle, knitr, testthat
LazyData: FALSE
RoxygenNote: 7.3.2
Encoding: UTF-8
URL: https://github.com/MetaboComp/batchCorr
BugReports: https://github.com/MetaboComp/batchCorr/issues
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/batchCorr
git_branch: devel
git_last_commit: e89ddb8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 20:38:24 UTC; biocbuild
Author: Anton Ribbenstedt [cre] (ORCID:
    <https://orcid.org/0000-0002-9985-5644>),
  Carl Brunius [aut] (ORCID: <https://orcid.org/0000-0003-3957-870X>),
  Vilhelm Suksi [aut]
Maintainer: Anton Ribbenstedt <parasitetwin@gmail.com>
