# Generated by roxygen2: do not edit by hand

export(alignBatches)
export(correctDrift)
export(getBatch)
export(mergeBatches)
export(normalizeBatches)
export(peakInfo)
exportMethods(alignBatches)
exportMethods(correctDrift)
exportMethods(normalizeBatches)
import(SummarizedExperiment)
import(mclust)
import(methods)
import(reshape)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rgb)
importFrom(graphics,hist)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,matplot)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(stats,aggregate)
importFrom(stats,dist)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,median)
importFrom(stats,predict)
importFrom(stats,smooth.spline)
